import os
from wotclientdetection.entities.launcher import LauncherBase, Launcher, LauncherStandalone
from wotclientdetection.constants import *

class LauncherManager:

    def __init__(self):
        self.__inititalized = False
        self.__launchers = []
        self.initialize()

    @property
    def initialized(self):
        return self.__inititalized

    def initialize(self):
        if self.__inititalized:
            return
        self.enumerate_launchers()
        self.__inititalized = True

    def finalize(self):
        if not self.__inititalized:
            return
        self.__launchers.clear()
        self.__inititalized = False

    def enumerate_launchers(self, force=False):
        if force:
            self.__launchers.clear()
        if len(self.__launchers):
            return
        for metadata in LAUNCHERS_METADATA:
            if metadata.flavour == LauncherFlavour.STANDALONE:
                self.__launchers.append(LauncherStandalone(metadata, ''))
                continue
            path = os.path.normpath(os.path.join(os.path.expandvars('%ProgramData%'), metadata.path))
            if not os.path.isdir(path):
                continue
            pointer_file_path = os.path.normpath(os.path.join(path, metadata.pointer))
            if not os.path.isfile(pointer_file_path):
                continue
            try:
                with open(pointer_file_path, 'r', newline='\n') as pointerFile:
                    real_path = os.path.normpath(pointerFile.read().strip())
            except:
                continue
            executable_path = os.path.normpath(os.path.join(real_path, metadata.executable))
            if not os.path.isfile(executable_path):
                continue
            launcher = Launcher(metadata, real_path)
            self.__launchers.append(launcher)

    def get_launcher(self, flavour) -> Launcher | LauncherStandalone:
        if not self.__inititalized:
            return None
        for launcher in self.__launchers:
            if launcher.flavour == flavour:
                return launcher
        return None

    def get_launchers(self) -> list[Launcher | LauncherStandalone]:
        if not self.__inititalized:
            return None
        return self.__launchers

    def remove_launcher(self, flavour):
        if not self.__inititalized:
            return
        launcher = self.get_launcher(flavour)
        if launcher is None:
            return
        self.__launchers.remove(launcher)



import collections

EXECUTABLE_NAME = 'WorldOfTanks.exe'

class LauncherFlavour:
    UNKNOWN = 0
    WG = 1
    CHINA_360 = 2
    STEAM = 3
    LESTA = 4
    STANDALONE = 5
    DEFAULT = WG

_LauncherMetadata = collections.namedtuple('LauncherMetadata', ('flavour', 'path', 'pointer', 'executable'))

LAUNCHERS_METADATA = (
    _LauncherMetadata(LauncherFlavour.WG, 'Wargaming.net\\GameCenter', 'data\\wgc_path.dat', 'wgc.exe'),
    _LauncherMetadata(LauncherFlavour.CHINA_360, '360 Wargaming\\GameCenter', 'data\\wgc_path.dat', 'wgc.exe'),
    _LauncherMetadata(LauncherFlavour.STEAM, 'Wargaming.net\\GameCenter for Steam', 'data\\wgc_path.dat', 'wgc.exe'),
    _LauncherMetadata(LauncherFlavour.LESTA, 'Lesta\\GameCenter', 'data\\lgc_path.dat', 'lgc.exe'),
    _LauncherMetadata(LauncherFlavour.STANDALONE, '', '', '')
)

class ClientBranch:
    UNKNOWN = 0
    RELEASE = 1
    COMMON_TEST = 2
    SUPERTEST = 3
    SANDBOX = 4


class ClientType:
    UNKNOWN = 0
    SD = 1
    HD = 2


class ClientRealm:
    EU = 'EU'
    NA = 'NA'
    ASIA = 'ASIA'
    CT = 'CT'
    CN = 'CN'
    RU = 'RU'
    RPT = 'RPT'



import os
import subprocess
import xml.etree.ElementTree as ET
import psutil
from wotclientdetection.entities.client import Client

class LauncherBase:

    def __init__(self, metadata, path):
        self._clients = []
        self.metadata = metadata
        self.path = path
        self.enumerate_clients()

    @property
    def flavour(self):
        return self.metadata.flavour

    @property
    def executable(self):
        return self.metadata.executable

    def enumerate_clients(self):
        self._clients.clear()

    def register_client(self, path, isPreffered=False):
        if self.is_registered(path):
            return
        client = Client(path, self.flavour, isPreffered)
        if not client.is_valid:
            return
        self._clients.append(client)

    def get_clients(self) -> list[Client]:
        return self._clients

    def get_client(self, branch=None, realm=None) -> Client:
        for client in self._clients:
            if branch is not None and branch == client.branch:
                return client
            if realm is not None and realm == client.realm:
                return client
        return None

    def get_preffered_client(self) -> Client:
        for client in self._clients:
            if client.is_preffered:
                return client
        return None

    def is_registered(self, path):
        for client in self._clients:
            if client.path == path:
                return True
        return False


class LauncherStandalone(LauncherBase):

    def __init__(self, metadata, path):
        super(LauncherStandalone, self).__init__(metadata, path)

    def __repr__(self):
        return (f'<LauncherStandalone metadata={self.metadata} path={self.path}>')


class Launcher(LauncherBase):

    def __init__(self, metadata, path):
        super(Launcher, self).__init__(metadata, path)

    def enumerate_clients(self):
        super(Launcher, self).enumerate_clients()
        preferences_path = os.path.normpath(os.path.join(self.path, 'preferences.xml'))
        if not os.path.isfile(preferences_path):
            return
        preferences_xml = ET.parse(preferences_path)
        root = preferences_xml.getroot()
        self.__get_preffered_client_from_preferences(root)
        self.__get_clients_from_preferences(root)
        self.__get_clients_from_apps()

    def is_started(self):
        for process in psutil.process_iter():
            if process is None:
                continue
            if process.name() != self.executable:
                continue
            if self.path in process.cwd():
                return True
        return False

    def run(self):
        executable_path = os.path.normpath(os.path.join(self.path, self.executable))
        if not os.path.isfile(executable_path):
            return
        try:
            subprocess.Popen(executable_path, shell=True)
            return True
        except:
            pass
        return False

    def terminate(self):
        for process in psutil.process_iter():
            if process is None:
                continue
            if process.name() != self.executable:
                continue
            if self.path in process.cwd():
                process.terminate()
                return True
        return False

    def __get_preffered_client_from_preferences(self, root):
        element = root.find('application/games_manager/selectedGames/WOT')
        if element is None:
            return
        path = element.text.strip()
        self.register_client(path, isPreffered=True)

    def __get_clients_from_preferences(self, root):
        elements = root.findall('application/games_manager/games/game')
        if elements is None:
            return
        for element in elements:
            child = element.find('working_dir')
            if child is None:
                continue
            path = child.text.strip()
            self.register_client(path)

    def __get_clients_from_apps(self):
        apps_path = os.path.normpath(os.path.join(self.path, 'apps'))
        for root, _, files in os.walk(apps_path):
            for file in files:
                if 'wot.' not in root:
                    continue
                manifestPath = os.path.normpath(os.path.join(root, file))
                try:
                    with open(manifestPath, 'r', newline='\n') as manifestFile:
                        clientPath = manifestFile.read().strip()
                except Exception:
                    pass
                self.register_client(clientPath)

    def __repr__(self):
        return (f'<Launcher metadata={self.metadata} path={self.path}>')



__title__ = 'wotclientdetection'
__version__ = '2024.07.28.1'

from wotclientdetection.manager import LauncherManager
from wotclientdetection.constants import LauncherFlavour, ClientBranch, ClientRealm

__all__ = (
    'LauncherManager',
    'LauncherFlavour',
    'ClientBranch',
    'ClientBranch',
    'ClientRealm',
)



import os
import re
import subprocess
import xml.etree.ElementTree as ET
import psutil
from wotclientdetection.constants import *

class Client:

    def __init__(self, path, launcher_flavour, is_preffered=False):
        self.branch = ClientBranch.UNKNOWN
        self.launcher_flavour = launcher_flavour
        self.l10n = None
        self.path = path
        self.path_mods = None
        self.path_resmods = None
        self.realm = None
        self.type = ClientType.UNKNOWN
        self.client_version = None
        self.exe_version = None
        self.is_preffered = is_preffered
        self.is_valid = self.__is_valid()
        self.__read_client_data()

    def update(self):
        self.__invalidate()
        self.__read_client_data()

    def is_version_match(self, pattern):
        regex = re.compile(pattern)
        match = regex.match(self.client_version)
        return bool(match)

    def is_started(self):
        for process in psutil.process_iter():
            if process is None:
                continue
            if process.name() != EXECUTABLE_NAME:
                continue
            if self.path in process.cwd():
                return True
        return False

    def run(self):
        executable_path = os.path.normpath(os.path.join(self.path, EXECUTABLE_NAME))
        if not os.path.isfile(executable_path):
            return
        try:
            subprocess.Popen(executable_path, shell=True)
            return True
        except:
            pass
        return False

    def terminate(self):
        for process in psutil.process_iter():
            if process is None:
                continue
            if process.name() != EXECUTABLE_NAME:
                continue
            if self.path in process.cwd():
                process.terminate()
                return True
        return False

    def __read_client_data(self):
        if not self.is_valid:
            return
        self.__invalidate()
        self.__read_app_type()
        self.__read_exe_version()
        self.__read_version()
        self.__read_game_info()
        self.__read_paths()

    def __read_app_type(self):
        app_type_path = os.path.normpath(os.path.join(self.path, 'app_type.xml'))
        if not os.path.isfile(app_type_path):
            return
        app_type_xml = ET.parse(app_type_path)
        root = app_type_xml.getroot()
        element = root.find('app_type')
        if element is None:
            return
        app_type = element.text.strip().lower()
        if app_type == 'hd':
            self.type = ClientType.HD
        elif app_type == 'sd':
            self.type = ClientType.SD

    def __read_exe_version(self):
        pass

    def __read_version(self):
        version_path = os.path.normpath(os.path.join(self.path, 'version.xml'))
        if not os.path.isfile(version_path):
            return
        version_xml = ET.parse(version_path)
        root = version_xml.getroot()
        element = root.find('meta/realm')
        if element is not None:
            self.realm = element.text.strip()
        element = root.find('version')
        if element is None:
            return
        version = element.text.strip()
        version = version.replace('v.', '')
        version = version.split()[:-1]
        version = ' '.join(version)
        version = version.split(None, 1)
        self.client_version = version[0]
        if len(version) == 2:
            branch = version[1]
            if branch == 'Common Test':
                self.branch = ClientBranch.COMMON_TEST
            elif branch == 'ST':
                self.branch = ClientBranch.SUPERTEST
            elif branch == 'SB':
                self.branch = ClientBranch.SANDBOX
            return
        if self.branch == ClientBranch.UNKNOWN:
            self.branch = ClientBranch.RELEASE

    def __read_game_info(self):
        game_info_path = os.path.normpath(os.path.join(self.path, 'game_info.xml'))
        if not os.path.isfile(game_info_path):
            return
        game_info_xml = ET.parse(game_info_path)
        root = game_info_xml.getroot()
        element = root.find('game/id')
        if element is not None:
            id = element.text.strip()
            if '.RPT.' in id:
                self.branch = ClientBranch.COMMON_TEST
        element = root.find('game/localization')
        if element is None:
            return
        self.l10n = element.text.strip()

    def __read_paths(self):
        paths_path = os.path.normpath(os.path.join(self.path, 'paths.xml'))
        if not os.path.isfile(paths_path):
            return
        paths_xml = ET.parse(paths_path)
        root = paths_xml.getroot()
        elements = root.findall('Paths/Path')
        if elements is None:
            return
        for element in elements:
            path = element.text.strip()
            path = path.replace('./', '')
            path = path.replace('/', '\\')
            if path.startswith('res_mods'):
                self.path_resmods = path
            elif path.startswith('mods'):
                self.path_mods = path

    def __is_valid(self):
        if not os.path.isdir(self.path):
            return False
        for file in ('app_type.xml', 'game_info.xml', 'paths.xml', 'version.xml', EXECUTABLE_NAME):
            if not os.path.isfile(os.path.join(self.path, file)):
                return False
        return True

    def __invalidate(self):
        self.branch = ClientBranch.UNKNOWN
        self.l10n = None
        self.path_mods = None
        self.path_resmods = None
        self.realm = None
        self.type = ClientType.UNKNOWN
        self.client_version = None
        self.exe_version = None

    def __repr__(self):
        return (f'<Client branch={self.branch} launcherFlavour={self.launcher_flavour} l10n={self.l10n} path={self.path} pathMods={self.path_mods} pathResmods={self.path_resmods} realm={self.realm} type={self.type} clientVersion={self.client_version} exeVersion={self.exe_version} isPreffered={self.is_preffered}>')
